# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Job.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'JobStatus' not in _M_omero.model.__dict__:
    _M_omero.model._t_JobStatus = IcePy.declareClass('::omero::model::JobStatus')
    _M_omero.model._t_JobStatusPrx = IcePy.declareProxy('::omero::model::JobStatus')

if 'JobOriginalFileLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_JobOriginalFileLink = IcePy.declareClass('::omero::model::JobOriginalFileLink')
    _M_omero.model._t_JobOriginalFileLinkPrx = IcePy.declareProxy('::omero::model::JobOriginalFileLink')

if 'OriginalFile' not in _M_omero.model.__dict__:
    _M_omero.model._t_OriginalFile = IcePy.declareClass('::omero::model::OriginalFile')
    _M_omero.model._t_OriginalFilePrx = IcePy.declareProxy('::omero::model::OriginalFile')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_JobOriginalFileLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_JobOriginalFileLinksSeq = IcePy.defineSequence('::omero::model::JobOriginalFileLinksSeq', (), _M_omero.model._t_JobOriginalFileLink)

if '_t_JobLinkedOriginalFileSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_JobLinkedOriginalFileSeq = IcePy.defineSequence('::omero::model::JobLinkedOriginalFileSeq', (), _M_omero.model._t_OriginalFile)

if 'Job' not in _M_omero.model.__dict__:
    _M_omero.model.Job = Ice.createTempClass()
    class Job(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _username=None, _groupname=None, _type=None, _message=None, _status=None, _submitted=None, _scheduledFor=None, _started=None, _finished=None, _originalFileLinksSeq=None, _originalFileLinksLoaded=False, _originalFileLinksCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.Job:
                raise RuntimeError('omero.model.Job is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._username = _username
            self._groupname = _groupname
            self._type = _type
            self._message = _message
            self._status = _status
            self._submitted = _submitted
            self._scheduledFor = _scheduledFor
            self._started = _started
            self._finished = _finished
            self._originalFileLinksSeq = _originalFileLinksSeq
            self._originalFileLinksLoaded = _originalFileLinksLoaded
            self._originalFileLinksCountPerOwner = _originalFileLinksCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Job')

        def ice_id(self, current=None):
            return '::omero::model::Job'

        def ice_staticId():
            return '::omero::model::Job'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getUsername(self, current=None):
            pass

        def setUsername(self, theUsername, current=None):
            pass

        def getGroupname(self, current=None):
            pass

        def setGroupname(self, theGroupname, current=None):
            pass

        def getType(self, current=None):
            pass

        def setType(self, theType, current=None):
            pass

        def getMessage(self, current=None):
            pass

        def setMessage(self, theMessage, current=None):
            pass

        def getStatus(self, current=None):
            pass

        def setStatus(self, theStatus, current=None):
            pass

        def getSubmitted(self, current=None):
            pass

        def setSubmitted(self, theSubmitted, current=None):
            pass

        def getScheduledFor(self, current=None):
            pass

        def setScheduledFor(self, theScheduledFor, current=None):
            pass

        def getStarted(self, current=None):
            pass

        def setStarted(self, theStarted, current=None):
            pass

        def getFinished(self, current=None):
            pass

        def setFinished(self, theFinished, current=None):
            pass

        def unloadOriginalFileLinks(self, current=None):
            pass

        def sizeOfOriginalFileLinks(self, current=None):
            pass

        def copyOriginalFileLinks(self, current=None):
            pass

        def addJobOriginalFileLink(self, target, current=None):
            pass

        def addAllJobOriginalFileLinkSet(self, targets, current=None):
            pass

        def removeJobOriginalFileLink(self, theTarget, current=None):
            pass

        def removeAllJobOriginalFileLinkSet(self, targets, current=None):
            pass

        def clearOriginalFileLinks(self, current=None):
            pass

        def reloadOriginalFileLinks(self, toCopy, current=None):
            pass

        def getOriginalFileLinksCountPerOwner(self, current=None):
            pass

        def linkOriginalFile(self, addition, current=None):
            pass

        def addJobOriginalFileLinkToBoth(self, link, bothSides, current=None):
            pass

        def findJobOriginalFileLink(self, removal, current=None):
            pass

        def unlinkOriginalFile(self, removal, current=None):
            pass

        def removeJobOriginalFileLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedOriginalFileList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Job)

        __repr__ = __str__

    _M_omero.model.JobPrx = Ice.createTempClass()
    class JobPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Job._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Job._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Job._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Job._op_setVersion.end(self, _r)

        def getUsername(self, _ctx=None):
            return _M_omero.model.Job._op_getUsername.invoke(self, ((), _ctx))

        def begin_getUsername(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getUsername.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getUsername(self, _r):
            return _M_omero.model.Job._op_getUsername.end(self, _r)

        def setUsername(self, theUsername, _ctx=None):
            return _M_omero.model.Job._op_setUsername.invoke(self, ((theUsername, ), _ctx))

        def begin_setUsername(self, theUsername, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setUsername.begin(self, ((theUsername, ), _response, _ex, _sent, _ctx))

        def end_setUsername(self, _r):
            return _M_omero.model.Job._op_setUsername.end(self, _r)

        def getGroupname(self, _ctx=None):
            return _M_omero.model.Job._op_getGroupname.invoke(self, ((), _ctx))

        def begin_getGroupname(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getGroupname.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getGroupname(self, _r):
            return _M_omero.model.Job._op_getGroupname.end(self, _r)

        def setGroupname(self, theGroupname, _ctx=None):
            return _M_omero.model.Job._op_setGroupname.invoke(self, ((theGroupname, ), _ctx))

        def begin_setGroupname(self, theGroupname, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setGroupname.begin(self, ((theGroupname, ), _response, _ex, _sent, _ctx))

        def end_setGroupname(self, _r):
            return _M_omero.model.Job._op_setGroupname.end(self, _r)

        def getType(self, _ctx=None):
            return _M_omero.model.Job._op_getType.invoke(self, ((), _ctx))

        def begin_getType(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getType.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getType(self, _r):
            return _M_omero.model.Job._op_getType.end(self, _r)

        def setType(self, theType, _ctx=None):
            return _M_omero.model.Job._op_setType.invoke(self, ((theType, ), _ctx))

        def begin_setType(self, theType, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setType.begin(self, ((theType, ), _response, _ex, _sent, _ctx))

        def end_setType(self, _r):
            return _M_omero.model.Job._op_setType.end(self, _r)

        def getMessage(self, _ctx=None):
            return _M_omero.model.Job._op_getMessage.invoke(self, ((), _ctx))

        def begin_getMessage(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getMessage.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMessage(self, _r):
            return _M_omero.model.Job._op_getMessage.end(self, _r)

        def setMessage(self, theMessage, _ctx=None):
            return _M_omero.model.Job._op_setMessage.invoke(self, ((theMessage, ), _ctx))

        def begin_setMessage(self, theMessage, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setMessage.begin(self, ((theMessage, ), _response, _ex, _sent, _ctx))

        def end_setMessage(self, _r):
            return _M_omero.model.Job._op_setMessage.end(self, _r)

        def getStatus(self, _ctx=None):
            return _M_omero.model.Job._op_getStatus.invoke(self, ((), _ctx))

        def begin_getStatus(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getStatus.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getStatus(self, _r):
            return _M_omero.model.Job._op_getStatus.end(self, _r)

        def setStatus(self, theStatus, _ctx=None):
            return _M_omero.model.Job._op_setStatus.invoke(self, ((theStatus, ), _ctx))

        def begin_setStatus(self, theStatus, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setStatus.begin(self, ((theStatus, ), _response, _ex, _sent, _ctx))

        def end_setStatus(self, _r):
            return _M_omero.model.Job._op_setStatus.end(self, _r)

        def getSubmitted(self, _ctx=None):
            return _M_omero.model.Job._op_getSubmitted.invoke(self, ((), _ctx))

        def begin_getSubmitted(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getSubmitted.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSubmitted(self, _r):
            return _M_omero.model.Job._op_getSubmitted.end(self, _r)

        def setSubmitted(self, theSubmitted, _ctx=None):
            return _M_omero.model.Job._op_setSubmitted.invoke(self, ((theSubmitted, ), _ctx))

        def begin_setSubmitted(self, theSubmitted, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setSubmitted.begin(self, ((theSubmitted, ), _response, _ex, _sent, _ctx))

        def end_setSubmitted(self, _r):
            return _M_omero.model.Job._op_setSubmitted.end(self, _r)

        def getScheduledFor(self, _ctx=None):
            return _M_omero.model.Job._op_getScheduledFor.invoke(self, ((), _ctx))

        def begin_getScheduledFor(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getScheduledFor.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getScheduledFor(self, _r):
            return _M_omero.model.Job._op_getScheduledFor.end(self, _r)

        def setScheduledFor(self, theScheduledFor, _ctx=None):
            return _M_omero.model.Job._op_setScheduledFor.invoke(self, ((theScheduledFor, ), _ctx))

        def begin_setScheduledFor(self, theScheduledFor, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setScheduledFor.begin(self, ((theScheduledFor, ), _response, _ex, _sent, _ctx))

        def end_setScheduledFor(self, _r):
            return _M_omero.model.Job._op_setScheduledFor.end(self, _r)

        def getStarted(self, _ctx=None):
            return _M_omero.model.Job._op_getStarted.invoke(self, ((), _ctx))

        def begin_getStarted(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getStarted.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getStarted(self, _r):
            return _M_omero.model.Job._op_getStarted.end(self, _r)

        def setStarted(self, theStarted, _ctx=None):
            return _M_omero.model.Job._op_setStarted.invoke(self, ((theStarted, ), _ctx))

        def begin_setStarted(self, theStarted, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setStarted.begin(self, ((theStarted, ), _response, _ex, _sent, _ctx))

        def end_setStarted(self, _r):
            return _M_omero.model.Job._op_setStarted.end(self, _r)

        def getFinished(self, _ctx=None):
            return _M_omero.model.Job._op_getFinished.invoke(self, ((), _ctx))

        def begin_getFinished(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getFinished.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getFinished(self, _r):
            return _M_omero.model.Job._op_getFinished.end(self, _r)

        def setFinished(self, theFinished, _ctx=None):
            return _M_omero.model.Job._op_setFinished.invoke(self, ((theFinished, ), _ctx))

        def begin_setFinished(self, theFinished, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_setFinished.begin(self, ((theFinished, ), _response, _ex, _sent, _ctx))

        def end_setFinished(self, _r):
            return _M_omero.model.Job._op_setFinished.end(self, _r)

        def unloadOriginalFileLinks(self, _ctx=None):
            return _M_omero.model.Job._op_unloadOriginalFileLinks.invoke(self, ((), _ctx))

        def begin_unloadOriginalFileLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_unloadOriginalFileLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadOriginalFileLinks(self, _r):
            return _M_omero.model.Job._op_unloadOriginalFileLinks.end(self, _r)

        def sizeOfOriginalFileLinks(self, _ctx=None):
            return _M_omero.model.Job._op_sizeOfOriginalFileLinks.invoke(self, ((), _ctx))

        def begin_sizeOfOriginalFileLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_sizeOfOriginalFileLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfOriginalFileLinks(self, _r):
            return _M_omero.model.Job._op_sizeOfOriginalFileLinks.end(self, _r)

        def copyOriginalFileLinks(self, _ctx=None):
            return _M_omero.model.Job._op_copyOriginalFileLinks.invoke(self, ((), _ctx))

        def begin_copyOriginalFileLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_copyOriginalFileLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyOriginalFileLinks(self, _r):
            return _M_omero.model.Job._op_copyOriginalFileLinks.end(self, _r)

        def addJobOriginalFileLink(self, target, _ctx=None):
            return _M_omero.model.Job._op_addJobOriginalFileLink.invoke(self, ((target, ), _ctx))

        def begin_addJobOriginalFileLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_addJobOriginalFileLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addJobOriginalFileLink(self, _r):
            return _M_omero.model.Job._op_addJobOriginalFileLink.end(self, _r)

        def addAllJobOriginalFileLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Job._op_addAllJobOriginalFileLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllJobOriginalFileLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_addAllJobOriginalFileLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllJobOriginalFileLinkSet(self, _r):
            return _M_omero.model.Job._op_addAllJobOriginalFileLinkSet.end(self, _r)

        def removeJobOriginalFileLink(self, theTarget, _ctx=None):
            return _M_omero.model.Job._op_removeJobOriginalFileLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeJobOriginalFileLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_removeJobOriginalFileLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeJobOriginalFileLink(self, _r):
            return _M_omero.model.Job._op_removeJobOriginalFileLink.end(self, _r)

        def removeAllJobOriginalFileLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Job._op_removeAllJobOriginalFileLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllJobOriginalFileLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_removeAllJobOriginalFileLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllJobOriginalFileLinkSet(self, _r):
            return _M_omero.model.Job._op_removeAllJobOriginalFileLinkSet.end(self, _r)

        def clearOriginalFileLinks(self, _ctx=None):
            return _M_omero.model.Job._op_clearOriginalFileLinks.invoke(self, ((), _ctx))

        def begin_clearOriginalFileLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_clearOriginalFileLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearOriginalFileLinks(self, _r):
            return _M_omero.model.Job._op_clearOriginalFileLinks.end(self, _r)

        def reloadOriginalFileLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Job._op_reloadOriginalFileLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadOriginalFileLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_reloadOriginalFileLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadOriginalFileLinks(self, _r):
            return _M_omero.model.Job._op_reloadOriginalFileLinks.end(self, _r)

        def getOriginalFileLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Job._op_getOriginalFileLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getOriginalFileLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_getOriginalFileLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getOriginalFileLinksCountPerOwner(self, _r):
            return _M_omero.model.Job._op_getOriginalFileLinksCountPerOwner.end(self, _r)

        def linkOriginalFile(self, addition, _ctx=None):
            return _M_omero.model.Job._op_linkOriginalFile.invoke(self, ((addition, ), _ctx))

        def begin_linkOriginalFile(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_linkOriginalFile.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkOriginalFile(self, _r):
            return _M_omero.model.Job._op_linkOriginalFile.end(self, _r)

        def addJobOriginalFileLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Job._op_addJobOriginalFileLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addJobOriginalFileLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_addJobOriginalFileLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addJobOriginalFileLinkToBoth(self, _r):
            return _M_omero.model.Job._op_addJobOriginalFileLinkToBoth.end(self, _r)

        def findJobOriginalFileLink(self, removal, _ctx=None):
            return _M_omero.model.Job._op_findJobOriginalFileLink.invoke(self, ((removal, ), _ctx))

        def begin_findJobOriginalFileLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_findJobOriginalFileLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findJobOriginalFileLink(self, _r):
            return _M_omero.model.Job._op_findJobOriginalFileLink.end(self, _r)

        def unlinkOriginalFile(self, removal, _ctx=None):
            return _M_omero.model.Job._op_unlinkOriginalFile.invoke(self, ((removal, ), _ctx))

        def begin_unlinkOriginalFile(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_unlinkOriginalFile.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkOriginalFile(self, _r):
            return _M_omero.model.Job._op_unlinkOriginalFile.end(self, _r)

        def removeJobOriginalFileLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Job._op_removeJobOriginalFileLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeJobOriginalFileLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_removeJobOriginalFileLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeJobOriginalFileLinkFromBoth(self, _r):
            return _M_omero.model.Job._op_removeJobOriginalFileLinkFromBoth.end(self, _r)

        def linkedOriginalFileList(self, _ctx=None):
            return _M_omero.model.Job._op_linkedOriginalFileList.invoke(self, ((), _ctx))

        def begin_linkedOriginalFileList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Job._op_linkedOriginalFileList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedOriginalFileList(self, _r):
            return _M_omero.model.Job._op_linkedOriginalFileList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.JobPrx.ice_checkedCast(proxy, '::omero::model::Job', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.JobPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Job'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_JobPrx = IcePy.defineProxy('::omero::model::Job', JobPrx)

    _M_omero.model._t_Job = IcePy.declareClass('::omero::model::Job')

    _M_omero.model._t_Job = IcePy.defineClass('::omero::model::Job', Job, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_username', (), _M_omero._t_RString, False, 0),
        ('_groupname', (), _M_omero._t_RString, False, 0),
        ('_type', (), _M_omero._t_RString, False, 0),
        ('_message', (), _M_omero._t_RString, False, 0),
        ('_status', (), _M_omero.model._t_JobStatus, False, 0),
        ('_submitted', (), _M_omero._t_RTime, False, 0),
        ('_scheduledFor', (), _M_omero._t_RTime, False, 0),
        ('_started', (), _M_omero._t_RTime, False, 0),
        ('_finished', (), _M_omero._t_RTime, False, 0),
        ('_originalFileLinksSeq', (), _M_omero.model._t_JobOriginalFileLinksSeq, False, 0),
        ('_originalFileLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_originalFileLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    Job._ice_type = _M_omero.model._t_Job

    Job._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Job._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Job._op_getUsername = IcePy.Operation('getUsername', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Job._op_setUsername = IcePy.Operation('setUsername', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Job._op_getGroupname = IcePy.Operation('getGroupname', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Job._op_setGroupname = IcePy.Operation('setGroupname', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Job._op_getType = IcePy.Operation('getType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Job._op_setType = IcePy.Operation('setType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Job._op_getMessage = IcePy.Operation('getMessage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Job._op_setMessage = IcePy.Operation('setMessage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Job._op_getStatus = IcePy.Operation('getStatus', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_JobStatus, False, 0), ())
    Job._op_setStatus = IcePy.Operation('setStatus', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobStatus, False, 0),), (), None, ())
    Job._op_getSubmitted = IcePy.Operation('getSubmitted', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RTime, False, 0), ())
    Job._op_setSubmitted = IcePy.Operation('setSubmitted', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RTime, False, 0),), (), None, ())
    Job._op_getScheduledFor = IcePy.Operation('getScheduledFor', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RTime, False, 0), ())
    Job._op_setScheduledFor = IcePy.Operation('setScheduledFor', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RTime, False, 0),), (), None, ())
    Job._op_getStarted = IcePy.Operation('getStarted', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RTime, False, 0), ())
    Job._op_setStarted = IcePy.Operation('setStarted', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RTime, False, 0),), (), None, ())
    Job._op_getFinished = IcePy.Operation('getFinished', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RTime, False, 0), ())
    Job._op_setFinished = IcePy.Operation('setFinished', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RTime, False, 0),), (), None, ())
    Job._op_unloadOriginalFileLinks = IcePy.Operation('unloadOriginalFileLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Job._op_sizeOfOriginalFileLinks = IcePy.Operation('sizeOfOriginalFileLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Job._op_copyOriginalFileLinks = IcePy.Operation('copyOriginalFileLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_JobOriginalFileLinksSeq, False, 0), ())
    Job._op_addJobOriginalFileLink = IcePy.Operation('addJobOriginalFileLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobOriginalFileLink, False, 0),), (), None, ())
    Job._op_addAllJobOriginalFileLinkSet = IcePy.Operation('addAllJobOriginalFileLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobOriginalFileLinksSeq, False, 0),), (), None, ())
    Job._op_removeJobOriginalFileLink = IcePy.Operation('removeJobOriginalFileLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobOriginalFileLink, False, 0),), (), None, ())
    Job._op_removeAllJobOriginalFileLinkSet = IcePy.Operation('removeAllJobOriginalFileLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobOriginalFileLinksSeq, False, 0),), (), None, ())
    Job._op_clearOriginalFileLinks = IcePy.Operation('clearOriginalFileLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Job._op_reloadOriginalFileLinks = IcePy.Operation('reloadOriginalFileLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Job, False, 0),), (), None, ())
    Job._op_getOriginalFileLinksCountPerOwner = IcePy.Operation('getOriginalFileLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Job._op_linkOriginalFile = IcePy.Operation('linkOriginalFile', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.model._t_JobOriginalFileLink, False, 0), ())
    Job._op_addJobOriginalFileLinkToBoth = IcePy.Operation('addJobOriginalFileLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobOriginalFileLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Job._op_findJobOriginalFileLink = IcePy.Operation('findJobOriginalFileLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.model._t_JobOriginalFileLinksSeq, False, 0), ())
    Job._op_unlinkOriginalFile = IcePy.Operation('unlinkOriginalFile', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), None, ())
    Job._op_removeJobOriginalFileLinkFromBoth = IcePy.Operation('removeJobOriginalFileLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_JobOriginalFileLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Job._op_linkedOriginalFileList = IcePy.Operation('linkedOriginalFileList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_JobLinkedOriginalFileSeq, False, 0), ())

    _M_omero.model.Job = Job
    del Job

    _M_omero.model.JobPrx = JobPrx
    del JobPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2013-2015 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt
# Sam Hart <sam@glencoesoftware.com>

"""
Tag plugin for command-line tag manipulation
"""
from __future__ import division

from builtins import input
from builtins import map
from builtins import str
from past.utils import old_div
from builtins import object
import builtins
import platform
import subprocess
import sys
import json

import omero
from omero.cli import BaseControl, CLI, ExceptionHandler
from omero.rtypes import rlong, rstring, unwrap
from omero.model import TagAnnotationI, AnnotationAnnotationLinkI

HELP = """Manage OMERO user tags.

Plugin for managing and viewing OMERO user tags.

Examples:

    omero tag list       # List all the tags, grouped by tagset
    omero tag create     # Creates a tag

    # Create a tag set named 'data_10.28' and associate the tag number 18 wth
    # it.
    omero tag createset --tag 18 --name data_10.28
"""


class Tag(object):
    def __init__(self, tag_id=None, name=None, description=None, owner=None,
                 children=None):
        self.tag_id = tag_id
        self.name = name
        self.description = description
        self.owner = owner
        self.children = children


class TagCollection(object):
    def __init__(self):
        self.tags = dict()
        self.owners = dict()
        self.mapping = dict()
        self.orphans = []
        self.empties = []


def exec_command(cmd):
    """
    given a command, will execute it in the parent environment
    Returns a list containing the output
    """
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
    output = p.stdout.readlines()
    p.stdout.close()
    return output


def clip(s, width):
    """
    Given a string, s, and a width, will clip the string to that width or
    fill it with spaces up to that width.

    Returns modified string
    """
    mod_s = s
    if s and len(s) > width:
        mod_s = s[:width]
    elif s and len(s) < width:
        mod_s = s + " " * (width - len(s))
    return mod_s


class TagControl(BaseControl):

    def _configure(self, parser):

        self.exc = ExceptionHandler()

        parser.add_login_arguments()
        sub = parser.sub()

        listtags = parser.add(
            sub, self.list, help="List all the tags, grouped by tagset")
        self.add_standard_params(listtags)
        listtags.add_argument(
            "--tagset", nargs="+", type=int, help="One or more tagset IDs")
        self.add_tag_common_params(listtags)
        listtags.add_login_arguments()

        listsets = parser.add(sub, self.listsets, help="List tag sets")
        self.add_standard_params(listsets)
        listsets.add_argument(
            "--tag", nargs="+", type=int,
            help="List only tagsets containing the following tag ID(s)")
        self.add_tag_common_params(listsets)
        listsets.add_login_arguments()

        create = parser.add(sub, self.create, help="Create a new tag")
        self.add_newtag_params(create)
        create.add_login_arguments()

        createset = parser.add(
            sub, self.createset, help="Create a new tag set")
        createset.add_argument(
            "--tag", nargs="+", required=True, type=int,
            help="ID(s) of the tag(s) to include in this set")
        self.add_newtag_params(createset)
        createset.add_login_arguments()

        loadj = sub.add_parser(
            self.load.__func__.__name__,
            help="Import new tag(s) and tagset(s) from JSON file",
            description="Import new tag(s) and tagset(s) from JSON file",
            epilog="""
JSON File Format:
  The format of the JSON file should be as follows:

  [{
    "name" : "Name of the tagset",
    "desc" : "Description of the tagset",
    "set" : [{
        "name" : "Name of tag",
        "desc" : "Description of tag"
    },{
        "name" : "Name of tag",
        "desc" : "Description of tag"
    },{
        ....
  },{
    ....
  }]
            """)
        loadj.set_defaults(func=self.load)
        loadj.add_argument(
            "filename", nargs="?", help="The filename containing tag JSON")
        loadj.add_login_arguments()

        links = parser.add(
            sub, self.link, help="Link annotation to an object")
        links.add_argument(
            "object",
            help="The object to link to. Should be of form"
            " <object_type>:<object_id>")
        links.add_argument(
            'tag_id', type=int,
            help="The tag annotation ID")
        self.add_standard_params(links)
        links.add_login_arguments()

    # Recurring parameter methods
    def add_newtag_params(self, parser):
        parser.add_argument(
            "--name", help="The name of the new tag or tagset")
        parser.add_argument(
            "--desc", "--description",
            help="The description of the new tag or tagset")

    def add_tag_common_params(self, parser):
        parser.add_argument(
            "--uid",
            help="List only tags/tagsets belonging to the following user ID")
        parser.add_argument(
            "--desc", "--description", "--descriptions",
            action="store_true", default=False,
            help="Display descriptions of tags")

    def add_standard_params(self, parser):
        parser.add_argument(
            "--admin", action="store_true", default=False,
            help="Perform action as an administrator")
        parser.add_argument(
            "--nopage", action="store_true", default=False,
            help="Disable pagination")

    # Output methods
    def print_line(self, line, index):
        if self.console_length is None:
                self.ctx.out(line)
        elif index % self.console_length == 0 and index:
            input_val = input("[Enter], [f]orward forever, or [q]uit: ")
            if input_val.lower() == 'q':
                sys.exit(0)
            elif input_val.lower() == 'f':
                self.console_length = None
        else:
            self.ctx.out(line)

    def pagetext(self, lines):
        for index, line in enumerate(lines):
            self.print_line(line, index)

    def pagetext_format(self, format, elements):
        for index, line in enumerate(elements):
            self.print_line(format.format(*line), index)

    def determine_console_size(self):
        """
        Will attempt to determine console size based upon the current
        platform.

        Returns tuple of width and length.
        """
        # The defaults if we can't figure it out
        lines = 25
        width = 80

        this_system = platform.system().lower()

        try:
            if this_system in ['linux', 'darwin', 'macosx', 'cygwin']:
                output = exec_command(['tput', 'lines'])
                if len(output) > 0:
                    lines = int(output[0].rstrip())
                output = exec_command(['tput', 'cols'])
                if len(output) > 0:
                    width = int(output[0].rstrip())
            elif this_system in ['windows', 'win32']:
                # http://stackoverflow.com/questions/566746/\
                # how-to-get-console-window-width-in-python
                from ctypes import windll, create_string_buffer
                from struct import unpack
                # stdin handle is -10
                # stdout handle is -11
                # stderr handle is -12
                h = windll.kernel32.GetStdHandle(-12)
                csbi = create_string_buffer(22)
                res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
                if res:
                    (bufx, bufy, curx, cury, wattr,
                     left, top, right, bottom,
                     maxx, maxy) = unpack("hhhhHhhhhhh", csbi.raw)
                    lines = bottom - top + 1
                    width = bottom - top + 1
        except:
            # Possible evil to ignore what the error was, but, truthfully,
            # the reason we do so is because it means it's a platform we
            # don't have support for, or a platform we should have support
            # for but which has some non-standard witch-craftery going on
            self.ctx.out("Could not determine the console length.")

        return width, lines

    # Data gathering methods
    def list_tags(self, args, tagset=None):
        """
        Returns a TagCollection object
        """
        # Get all tags except empty tagsets recursively
        tc = self.list_tags_recursive(args, tagset=tagset)

        # Now add the empty tagsets to the collection
        params = omero.sys.ParametersI()
        params.addString('ns', omero.constants.metadata.NSINSIGHTTAGSET)
        ice_map = dict()
        if args.admin:
            ice_map["omero.group"] = "-1"

        client = self.ctx.conn(args)
        session = client.getSession()
        q = session.getQueryService()

        sql = """
            select ann.id, ann.description, ann.textValue
            from TagAnnotation ann where ann.id not in
            (select distinct l.parent.id from AnnotationAnnotationLink l)
            and ann.ns=:ns
            """

        if args.uid:
            params.map["eid"] = rlong(int(args.uid))
            sql += " and ann.details.owner.id = :eid"

        if tagset:
            sql += " and ann.id = :tid"
            params.map['tid'] = rlong(int(tagset))

        for element in q.projection(sql, params, ice_map):
            tag_id, description, text = list(map(unwrap, element))
            tc.empties.append(Tag(
                tag_id=tag_id,
                name=text,
                description=description))

        return tc

    def list_tags_recursive(self, args, tagset=None):
        """
        Returns a TagCollection object
        """
        params = omero.sys.ParametersI()
        params.addString('ns', omero.constants.metadata.NSINSIGHTTAGSET)
        ice_map = dict()
        if args.admin:
            ice_map["omero.group"] = "-1"

        client = self.ctx.conn(args)
        session = client.getSession()
        q = session.getQueryService()

        parent_tags = []

        sql = """
            select aal.parent.id, aal.child.id, aal.parent.description,
            aal.parent.textValue from AnnotationAnnotationLink aal
            inner join aal.parent ann
            where ann.ns=:ns
            """
        if args.uid:
            params.map["eid"] = rlong(int(args.uid))
            sql += " and ann.details.owner.id = :eid"
        if tagset:
            sql += " and ann.id = :tid"
            params.map['tid'] = rlong(int(tagset))

        tc = TagCollection()

        for element in q.projection(sql, params, ice_map):
            parent = unwrap(element[0])
            child = unwrap(element[1])
            tc.mapping.setdefault(parent, []).append(child)
            parent_tags.append(Tag(
                tag_id=parent,
                name=unwrap(element[3]),
                description=unwrap(element[2])
            ))

        if tagset:
            sql = """
                select distinct child.id, child.description, child.textValue,
                owner.id, owner.firstName, owner.lastName
                from AnnotationAnnotationLink as ann
                join ann.child as child
                join child.details.owner as owner
                join ann.parent as parent
                where parent.id = :tid or child.id = :tid
                """
        else:
            # Get orphans first
            sql = """
                select ann.id, ann.textValue, ann.description
                from TagAnnotation ann where ann.id not in
                (select distinct l.child.id from AnnotationAnnotationLink l
                    join l.parent as ts
                    where ts.ns = :ns)
                and ann.ns is null
                """
            if args.uid:
                sql += " and ann.details.owner.id = :eid"

            for element in q.projection(sql, params, ice_map):
                tag_id, text, description = list(map(unwrap, element))
                tc.orphans.append(Tag(
                    tag_id=tag_id,
                    name=text,
                    description=description))

            # Now set up search for rest of tags
            sql = """
                select ann.id, ann.description, ann.textValue,
                ann.details.owner.id,
                ann.details.owner.firstName, ann.details.owner.lastName
                from TagAnnotation ann
                """

        if args.uid:
            if tagset:
                sql += " and ann.details.owner.id = :eid"
            else:
                sql += " where ann.details.owner.id = :eid"

        for element in q.projection(sql, params, ice_map):
            tag_id, description, text, owner, first, last = list(map(unwrap,
                                                                element))
            tc.tags[tag_id] = Tag(
                tag_id=tag_id,
                name=text,
                description=description,
                owner=owner,
                children=tc.mapping.get(tag_id) or 0
                )
            tc.owners[owner] = "%s %s" % (first, last)

        for parent in parent_tags:
            if parent.tag_id not in tc.tags:
                tc.tags[parent.tag_id] = parent

        return tc

    def list_tagsets(self, args, tag):
        """
        Returns a TagCollection of just the tagsets.

        If tag is provided, will return the tagsets with those tags.
        """
        params = omero.sys.ParametersI()
        params.addString('ns', omero.constants.metadata.NSINSIGHTTAGSET)
        ice_map = dict()
        if args.admin:
            ice_map["omero.group"] = "-1"

        client = self.ctx.conn(args)
        session = client.getSession()
        q = session.getQueryService()

        tc = TagCollection()

        if tag:
            sql = """
                select a.id, a.description, a.textValue,
                a.details.owner.id, a.details.owner.firstName,
                a.details.owner.lastName
                from AnnotationAnnotationLink b
                inner join b.parent a
                where a.ns=:ns
                """
            sql += " and b.child.id = :tid"
            params.map['tid'] = rlong(int(tag))
        else:
            sql = """
                select a.id, a.description, a.textValue,
                a.details.owner.id, a.details.owner.firstName,
                a.details.owner.lastName
                from TagAnnotation a
                where a.ns=:ns
                """

        if args.uid:
            params.map["eid"] = rlong(int(args.uid))
            sql += " and a.details.owner.id = :eid"

        for element in q.projection(sql, params, ice_map):
            tag_id, description, text, owner, first, last = list(map(unwrap,
                                                                element))
            tc.tags[tag_id] = Tag(
                tag_id=tag_id,
                name=text,
                description=description,
                owner=owner
            )
            tc.owners[owner] = "%s %s" % (first, last)

        return tc

    def generate_tagset(self, tags, mapping, args):
        """
        Given a dict of tags and mappings for parent/child relationships
        return a list of lines representing the tagset output.
        """
        lines = []
        for key in list(mapping.keys()):
            lines.append("+- %s:'%s'" % (str(key), tags[key].name))
            if args.desc:
                lines.append("|   '%s'" % tags[key].description)
                lines.append("|")
            lines.append('|\\')
            for tag_key in mapping[key]:
                lines.append("| +- %s:'%s'" % (str(tag_key),
                             tags[tag_key].name))
                if args.desc:
                    lines.append("|   '%s'" % tags[tag_key].description)
                    lines.append("|")
            lines.append('')

        return lines

    def generate_orphans(self, orphans, args):
        """
        Given a list of orphaned tags, return a list of
        lines representing the orphan output.
        """
        lines = []
        lines.append('Orphaned tags:')
        for orphan in orphans:
            lines.append("> %s:'%s'" % (str(orphan.tag_id), orphan.name))
            if args.desc:
                lines.append("   '%s'" % orphan.description)
                lines.append('')
        return lines

    def generate_empties(self, empties, args):
        """
        Given a list of empty tagsets, return a list of
        lines representing the empty tagset output.
        """
        lines = []
        lines.append('Empty tagsets:')
        for empty in empties:
            lines.append("> %s:'%s'" % (str(empty.tag_id), empty.name))
            if args.desc:
                lines.append("   '%s'" % empty.description)
                lines.append('')
        return lines

    def create_tag(self, name, description, text="tag"):
        """
        Creates a new tag object. Returns the new tag object.

        If name parameter is None, the user will be prompted to input it.

        The "text" parameter should be the text description to use upon user
        input. For example, if we were creating a tag, this would be "tag"
        (the default). If we were creating a tagset, this could be "tag set".
        """
        if name is None:
            name = builtins.input("Please enter a name for this %s: " % text)

        if name is not None and name != '':
            tag = TagAnnotationI()
            tag.textValue = rstring(name)
            if description is not None and len(description) > 0:
                tag.description = rstring(description)

            return tag
        else:
            self.ctx.err("Tag/tagset name cannot be 'None' or empty.")
            sys.exit(1)

    # Actual command methods
    def create(self, args):
        """
        create a tag command.
        """
        tag = self.create_tag(args.name, args.desc)

        client = self.ctx.conn(args)
        session = client.getSession()
        update_service = session.getUpdateService()
        tag = update_service.saveAndReturnObject(tag)
        self.ctx.out("TagAnnotation:%s" % tag.id.val)

    def createset(self, args):
        """
        Create a tag set command.
        """
        tags = []
        if args.tag:
            if type(args.tag) is list:
                tags = args.tag
            else:
                tags = [args.tag]
        else:
            # Should not happen
            self.ctx.err("Missing tag parameter")
            sys.exit(1)

        tag = self.create_tag(args.name, args.desc, text="tag set")
        tag.ns = rstring(omero.constants.metadata.NSINSIGHTTAGSET)
        links = []
        for t in tags:
            link = AnnotationAnnotationLinkI()
            link.parent = tag
            link.child = TagAnnotationI(rlong(int(t)), False)
            links.append(link)
        client = self.ctx.conn(args)
        session = client.getSession()
        update_service = session.getUpdateService()
        try:
            links = update_service.saveAndReturnArray(links)
            self.ctx.out("TagAnnotation:%s" % links[0].parent.id.val)
        except omero.ValidationException as e:
            self.ctx.err(e.message)
            self.ctx.err("Check that tag '%s' exists." % t)
            sys.exit(1)

    def load(self, args):
        """
        Import new tag(s) from json.
        """
        if args.filename:
            fobj = open(args.filename)
        else:
            fobj = sys.stdin

        p = json.load(fobj)

        if fobj is not sys.stdin:
            fobj.close()

        to_add = []

        for element in p:
            if 'set' in element:
                tag = self.create_tag(str(element['name']),
                                      str(element['desc']))
                tag.ns = rstring(omero.constants.metadata.NSINSIGHTTAGSET)
                links = []
                for e in element['set']:
                    t = self.create_tag(str(e['name']), str(e['desc']))
                    link = AnnotationAnnotationLinkI()
                    link.parent = tag
                    link.child = t
                    links.append(link)
                to_add.extend(links)
            else:
                to_add.append(self.create_tag(str(element['name']),
                              str(element['desc'])))

        client = self.ctx.conn(args)
        session = client.getSession()
        update_service = session.getUpdateService()
        to_add = update_service.saveAndReturnArray(to_add)
        ids = []
        for element in to_add:
            if isinstance(element, TagAnnotationI):
                self.ctx.out("TagAnnotation:%s" % element.id.val)
                ids.append(element.id.val)
            else:
                tag_id = element.parent.id.val
                if tag_id not in ids:
                    self.ctx.out("TagAnnotation:%s" % tag_id)
                    ids.append(tag_id)

    def link(self, args):
        """
        Links an object to a tag annotation.
        """

        try:
            obj_type, obj_id = args.object.split(':')
            obj_id = int(obj_id)
        except ValueError:
            obj_type = None
            obj_id = None

        if obj_type is None or obj_id is None:
            self.ctx.err("Missing object or object not of form"
                         " <object_type>:<object_id>")
            sys.exit(1)

        if not args.tag_id:
            self.ctx.err("Missing tag_id")
            sys.exit(1)

        tag_id = args.tag_id

        parameters = omero.sys.ParametersI()
        parameters.addId(obj_id)
        ice_map = dict()
        if args.admin:
            ice_map["omero.group"] = "-1"

        # Retrieve annotation
        client = self.ctx.conn(args)
        session = client.getSession()
        query_service = session.getQueryService()
        update_service = session.getUpdateService()
        try:
            annotation = query_service.find("TagAnnotation", tag_id)
        except omero.SecurityViolation as sv:
            self.ctx.die(510, "SecurityViolation: %s" % sv.message)

        if not annotation:
            self.ctx.die(400, "Could not find annotation")

        obj = query_service.findByQuery(
            "select o from %s as o "
            "left outer join fetch o.annotationLinks "
            "where o.id = :id" % obj_type, parameters, ice_map)
        if obj is None:
            self.ctx.err(
                "Object query returned nothing. Check your object type.")
            sys.exit(1)

        obj.linkAnnotation(annotation)
        try:
            obj = update_service.saveAndReturnObject(obj)
        except omero.SecurityViolation as sv:
            self.ctx.die(510, "SecurityViolation: %s" % sv.message)

        self.ctx.out("%sAnnotationLink:%s" % (obj_type, obj.id.val))

    def list(self, args):
        """
        List tags command.
        """
        if args.nopage:
            self.console_length = None
            self.width = 80
        else:
            self.width, self.console_length = self.determine_console_size()

        tagsets = [None]
        lines = []
        if args.tagset:
            if type(args.tagset) is list:
                tagsets = args.tagset
            else:
                tagsets = [args.tagset]

        for tagset in tagsets:
            tc = self.list_tags(args, tagset)
            lines.extend(self.generate_tagset(tc.tags, tc.mapping, args))
            if len(tc.orphans) > 0:
                lines.extend(self.generate_orphans(tc.orphans, args))
                if len(tc.empties) > 0:
                    lines.append('')
            if len(tc.empties) > 0:
                lines.extend(self.generate_empties(tc.empties, args))
        self.pagetext(lines)

    def listsets(self, args):
        """
        List tag sets command.
        """
        # The max width of the ID field. We need something here unless
        # we want to pre-search to determine the max size. Our assumption
        # here is that we wont have more than 10,000,000 tags.
        # FIXME - We can figure this out easily enough- Sam
        max_id_width = 8

        if args.nopage:
            self.console_length = None
            self.width = 80
        else:
            self.width, self.console_length = self.determine_console_size()

        if args.desc:
            max_field_width = int((old_div((self.width - max_id_width), 2.0)) - 2)
        else:
            max_field_width = self.width - max_id_width - 2

        tags = [None]
        lines = []
        if args.tag:
            if type(args.tag) is list:
                tags = args.tag
            else:
                tags = [args.tag]

        separator = (
            "-" * max_id_width,
            "-" * max_field_width,
            "-" * max_field_width
        )

        lines.append(separator)

        lines.append((
            clip("ID", max_id_width),
            clip("Name", max_field_width),
            clip("Description", max_field_width)
        ))

        lines.append(separator)

        for tag_id in tags:
            tc = self.list_tagsets(args, tag_id)
            for key, tag in list(tc.tags.items()):
                if args.nopage:
                    lines.append((
                        tag.tag_id,
                        tag.name,
                        tag.description

                    ))
                else:
                    lines.append((
                        clip(str(tag.tag_id), max_id_width),
                        clip(tag.name, max_field_width),
                        clip(tag.description, max_field_width)
                    ))

        lines.append(separator)

        if args.desc:
            self.pagetext_format("{0}|{1}|{2}", lines)
        else:
            self.pagetext_format("{0}|{1}", lines)


try:
    register("tag", TagControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("user", TagControl, HELP)
        cli.invoke(sys.argv[1:])



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `MicrobeamManipulation.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'MicrobeamManipulationType' not in _M_omero.model.__dict__:
    _M_omero.model._t_MicrobeamManipulationType = IcePy.declareClass('::omero::model::MicrobeamManipulationType')
    _M_omero.model._t_MicrobeamManipulationTypePrx = IcePy.declareProxy('::omero::model::MicrobeamManipulationType')

if 'LightSettings' not in _M_omero.model.__dict__:
    _M_omero.model._t_LightSettings = IcePy.declareClass('::omero::model::LightSettings')
    _M_omero.model._t_LightSettingsPrx = IcePy.declareProxy('::omero::model::LightSettings')

if 'Experiment' not in _M_omero.model.__dict__:
    _M_omero.model._t_Experiment = IcePy.declareClass('::omero::model::Experiment')
    _M_omero.model._t_ExperimentPrx = IcePy.declareProxy('::omero::model::Experiment')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_MicrobeamManipulationLightSourceSettingsSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_MicrobeamManipulationLightSourceSettingsSeq = IcePy.defineSequence('::omero::model::MicrobeamManipulationLightSourceSettingsSeq', (), _M_omero.model._t_LightSettings)

if 'MicrobeamManipulation' not in _M_omero.model.__dict__:
    _M_omero.model.MicrobeamManipulation = Ice.createTempClass()
    class MicrobeamManipulation(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _type=None, _lightSourceSettingsSeq=None, _lightSourceSettingsLoaded=False, _experiment=None, _description=None):
            if Ice.getType(self) == _M_omero.model.MicrobeamManipulation:
                raise RuntimeError('omero.model.MicrobeamManipulation is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._type = _type
            self._lightSourceSettingsSeq = _lightSourceSettingsSeq
            self._lightSourceSettingsLoaded = _lightSourceSettingsLoaded
            self._experiment = _experiment
            self._description = _description

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::MicrobeamManipulation')

        def ice_id(self, current=None):
            return '::omero::model::MicrobeamManipulation'

        def ice_staticId():
            return '::omero::model::MicrobeamManipulation'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getType(self, current=None):
            pass

        def setType(self, theType, current=None):
            pass

        def unloadLightSourceSettings(self, current=None):
            pass

        def sizeOfLightSourceSettings(self, current=None):
            pass

        def copyLightSourceSettings(self, current=None):
            pass

        def addLightSettings(self, target, current=None):
            pass

        def addAllLightSettingsSet(self, targets, current=None):
            pass

        def removeLightSettings(self, theTarget, current=None):
            pass

        def removeAllLightSettingsSet(self, targets, current=None):
            pass

        def clearLightSourceSettings(self, current=None):
            pass

        def reloadLightSourceSettings(self, toCopy, current=None):
            pass

        def getExperiment(self, current=None):
            pass

        def setExperiment(self, theExperiment, current=None):
            pass

        def getDescription(self, current=None):
            pass

        def setDescription(self, theDescription, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_MicrobeamManipulation)

        __repr__ = __str__

    _M_omero.model.MicrobeamManipulationPrx = Ice.createTempClass()
    class MicrobeamManipulationPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_setVersion.end(self, _r)

        def getType(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getType.invoke(self, ((), _ctx))

        def begin_getType(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getType.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getType(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_getType.end(self, _r)

        def setType(self, theType, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setType.invoke(self, ((theType, ), _ctx))

        def begin_setType(self, theType, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setType.begin(self, ((theType, ), _response, _ex, _sent, _ctx))

        def end_setType(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_setType.end(self, _r)

        def unloadLightSourceSettings(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_unloadLightSourceSettings.invoke(self, ((), _ctx))

        def begin_unloadLightSourceSettings(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_unloadLightSourceSettings.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadLightSourceSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_unloadLightSourceSettings.end(self, _r)

        def sizeOfLightSourceSettings(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_sizeOfLightSourceSettings.invoke(self, ((), _ctx))

        def begin_sizeOfLightSourceSettings(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_sizeOfLightSourceSettings.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfLightSourceSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_sizeOfLightSourceSettings.end(self, _r)

        def copyLightSourceSettings(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_copyLightSourceSettings.invoke(self, ((), _ctx))

        def begin_copyLightSourceSettings(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_copyLightSourceSettings.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyLightSourceSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_copyLightSourceSettings.end(self, _r)

        def addLightSettings(self, target, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_addLightSettings.invoke(self, ((target, ), _ctx))

        def begin_addLightSettings(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_addLightSettings.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addLightSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_addLightSettings.end(self, _r)

        def addAllLightSettingsSet(self, targets, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_addAllLightSettingsSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllLightSettingsSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_addAllLightSettingsSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllLightSettingsSet(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_addAllLightSettingsSet.end(self, _r)

        def removeLightSettings(self, theTarget, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_removeLightSettings.invoke(self, ((theTarget, ), _ctx))

        def begin_removeLightSettings(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_removeLightSettings.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeLightSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_removeLightSettings.end(self, _r)

        def removeAllLightSettingsSet(self, targets, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_removeAllLightSettingsSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllLightSettingsSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_removeAllLightSettingsSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllLightSettingsSet(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_removeAllLightSettingsSet.end(self, _r)

        def clearLightSourceSettings(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_clearLightSourceSettings.invoke(self, ((), _ctx))

        def begin_clearLightSourceSettings(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_clearLightSourceSettings.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearLightSourceSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_clearLightSourceSettings.end(self, _r)

        def reloadLightSourceSettings(self, toCopy, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_reloadLightSourceSettings.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadLightSourceSettings(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_reloadLightSourceSettings.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadLightSourceSettings(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_reloadLightSourceSettings.end(self, _r)

        def getExperiment(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getExperiment.invoke(self, ((), _ctx))

        def begin_getExperiment(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getExperiment.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getExperiment(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_getExperiment.end(self, _r)

        def setExperiment(self, theExperiment, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setExperiment.invoke(self, ((theExperiment, ), _ctx))

        def begin_setExperiment(self, theExperiment, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setExperiment.begin(self, ((theExperiment, ), _response, _ex, _sent, _ctx))

        def end_setExperiment(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_setExperiment.end(self, _r)

        def getDescription(self, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getDescription.invoke(self, ((), _ctx))

        def begin_getDescription(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_getDescription.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDescription(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_getDescription.end(self, _r)

        def setDescription(self, theDescription, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setDescription.invoke(self, ((theDescription, ), _ctx))

        def begin_setDescription(self, theDescription, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulation._op_setDescription.begin(self, ((theDescription, ), _response, _ex, _sent, _ctx))

        def end_setDescription(self, _r):
            return _M_omero.model.MicrobeamManipulation._op_setDescription.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.MicrobeamManipulationPrx.ice_checkedCast(proxy, '::omero::model::MicrobeamManipulation', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.MicrobeamManipulationPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::MicrobeamManipulation'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_MicrobeamManipulationPrx = IcePy.defineProxy('::omero::model::MicrobeamManipulation', MicrobeamManipulationPrx)

    _M_omero.model._t_MicrobeamManipulation = IcePy.declareClass('::omero::model::MicrobeamManipulation')

    _M_omero.model._t_MicrobeamManipulation = IcePy.defineClass('::omero::model::MicrobeamManipulation', MicrobeamManipulation, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_type', (), _M_omero.model._t_MicrobeamManipulationType, False, 0),
        ('_lightSourceSettingsSeq', (), _M_omero.model._t_MicrobeamManipulationLightSourceSettingsSeq, False, 0),
        ('_lightSourceSettingsLoaded', (), IcePy._t_bool, False, 0),
        ('_experiment', (), _M_omero.model._t_Experiment, False, 0),
        ('_description', (), _M_omero._t_RString, False, 0)
    ))
    MicrobeamManipulation._ice_type = _M_omero.model._t_MicrobeamManipulation

    MicrobeamManipulation._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    MicrobeamManipulation._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    MicrobeamManipulation._op_getType = IcePy.Operation('getType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_MicrobeamManipulationType, False, 0), ())
    MicrobeamManipulation._op_setType = IcePy.Operation('setType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_MicrobeamManipulationType, False, 0),), (), None, ())
    MicrobeamManipulation._op_unloadLightSourceSettings = IcePy.Operation('unloadLightSourceSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    MicrobeamManipulation._op_sizeOfLightSourceSettings = IcePy.Operation('sizeOfLightSourceSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    MicrobeamManipulation._op_copyLightSourceSettings = IcePy.Operation('copyLightSourceSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_MicrobeamManipulationLightSourceSettingsSeq, False, 0), ())
    MicrobeamManipulation._op_addLightSettings = IcePy.Operation('addLightSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_LightSettings, False, 0),), (), None, ())
    MicrobeamManipulation._op_addAllLightSettingsSet = IcePy.Operation('addAllLightSettingsSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_MicrobeamManipulationLightSourceSettingsSeq, False, 0),), (), None, ())
    MicrobeamManipulation._op_removeLightSettings = IcePy.Operation('removeLightSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_LightSettings, False, 0),), (), None, ())
    MicrobeamManipulation._op_removeAllLightSettingsSet = IcePy.Operation('removeAllLightSettingsSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_MicrobeamManipulationLightSourceSettingsSeq, False, 0),), (), None, ())
    MicrobeamManipulation._op_clearLightSourceSettings = IcePy.Operation('clearLightSourceSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    MicrobeamManipulation._op_reloadLightSourceSettings = IcePy.Operation('reloadLightSourceSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_MicrobeamManipulation, False, 0),), (), None, ())
    MicrobeamManipulation._op_getExperiment = IcePy.Operation('getExperiment', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Experiment, False, 0), ())
    MicrobeamManipulation._op_setExperiment = IcePy.Operation('setExperiment', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Experiment, False, 0),), (), None, ())
    MicrobeamManipulation._op_getDescription = IcePy.Operation('getDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    MicrobeamManipulation._op_setDescription = IcePy.Operation('setDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.MicrobeamManipulation = MicrobeamManipulation
    del MicrobeamManipulation

    _M_omero.model.MicrobeamManipulationPrx = MicrobeamManipulationPrx
    del MicrobeamManipulationPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2008-2014 Glencoe Software, Inc. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
   Plugin for viewing and controlling active sessions for a local user.

   Plugin read by omero.cli.Cli during initialization. The method(s)
   defined here will be added to the Cli class for later use.
"""
from __future__ import division

from builtins import str
from past.utils import old_div
import datetime
import os
import sys
import Ice
import IceImport
import time
import traceback
import warnings
import omero.java

IceImport.load("Glacier2_Router_ice")

from Glacier2 import PermissionDeniedException

from omero.rtypes import rlong
from omero.rtypes import unwrap
from omero.util import get_user
from omero.util.sessions import SessionsStore
from omero.cli import UserGroupControl, CLI, admin_only
from omero_ext.argparse import SUPPRESS
from omero.model.enums import AdminPrivilegeSudo

HELP = """Control and create user sessions

Sessions are stored locally on disk. Several can
be active simultaneously, but only one will be used
for a single invocation of omero.

"""

LONGHELP = """
Uses the login parameters from %(prog)s to login.

To list these options, use "%(prog)s -h"

Options for logging in:

    # Provide all values interactively
    $ omero sessions login
    Server: [localhost:4064]
    Username: [root]
    Password:

    # Pass values as the target
    $ omero sessions login user@omero.example.com
    Password:
    $ omero sessions login user@omero.example.com:24064
    Password:

    # Pass some values via arguments
    $ omero -s localhost sessions login
    Username: [user]
    Password:
    $ omero -p 24064 sessions login
    Server: [localhost:24064]
    Username: [user]
    Password:

    # Pass all non-password values via arguments
    $ omero -s localhost -u john sessions login
    Password:

    # Use a session ID to login without a password
    $ omero -s localhost -k 8afe443f-19fc-4cc4-bf4a-850ec94f4650 \
    sessions login

    # Arguments can also go earlier
    $ omero -k 8afe443f-19fc-4cc4-bf4a-850ec94f4650 sessions login

    # The *last* "@" symbol is used
    $ omero sessions login my.email@example.com@omero.example.com
    Password:

    # System administrators can use "--sudo" to login as others
    $ omero sessions login --sudo=root example@localhost
    Password for root:

Other sessions commands:

    # Logging out of the currently active sessions
    $ omero sessions logout

    # List all locally available sessions (purging the expired ones)
    $ omero sessions list

    # List all local sessions
    $ omero sessions list --no-purge

    # List all active server sessions
    $ omero sessions who

    # List or change the group for the session
    $ omero sessions group
    $ omero sessions group mygroup
    $ omero sessions group 123

    # List or change the timeToLive for the session
    $ omero sessions timeout
    $ omero sessions timeout 300 # Seconds
    $ omero sessions timeout 300 --session=$UUID

Custom sessions directory:

    # Specify a custom session directory using OMERO_SESSIONDIR
    $ export OMERO_SESSIONDIR=/tmp/my_sessions
    # Create a new session stored under OMERO_SESSIONDIR
    $ omero sessions login
    $ omero sessions file
    $ omero sessions list
"""

LISTHELP = """
By default, inactive sessions are purged from the local sessions store and
removed from the listing. To list all sessions stored locally independently of
their status, use the --no-purge argument.
"""

GROUPHELP = """
If any current services are open, the command will fail.
"""

WHOHELP = """
Administrators will receive a list of all active sessions
along with critical information on last activity. This is
useful for determining whether or not the server can be
restarted.

Other users will only see a list of names, i.e. users who
can be considered "online".
"""


class SessionsControl(UserGroupControl):

    FACTORY = SessionsStore

    def store(self, args):
        try:
            # Read base directory from deprecated --session-dir argument
            base_dir = getattr(args, "session_dir", None)
            if base_dir:
                warnings.warn(
                    "--session-dir is deprecated. Use OMERO_SESSIONDIR"
                    " instead.", DeprecationWarning)

            # Read base directory from deprecated OMERO_SESSION_DIR envvar
            base_dir = os.environ.get('OMERO_SESSION_DIR', base_dir)
            if 'OMERO_SESSION_DIR' in os.environ:
                warnings.warn(
                    "OMERO_SESSION_DIR is deprecated. Use OMERO_SESSIONDIR"
                    " instead.", DeprecationWarning)

            # Read sessions directory from OMERO_SESSIONDIR envvar
            session_dir = None
            if base_dir:
                from omero_ext.path import path
                session_dir = path(base_dir) / "omero" / "sessions"
            sessions_dir = os.environ.get('OMERO_SESSIONDIR', session_dir)

            return self.FACTORY(sessions_dir)
        except OSError as ose:
            filename = getattr(ose, "filename", sessions_dir)
            self.ctx.die(155, "Could not access session dir: %s" % filename)

    def _configure(self, parser):
        parser.add_login_arguments()
        sub = parser.sub()
        parser.add(sub, self.help, "Extended help")
        login = parser.add(
            sub, self.login, self.login.__doc__)
        logout = parser.add(
            sub, self.logout, "Logout and remove current session key")
        self._configure_login(login)

        group = parser.add(
            sub, self.group,
            "Set the group of the given session by id or name" + GROUPHELP)
        group.add_argument(
            "target",
            nargs="?",
            help="Id or name of the group to switch this session to")

        timeout = parser.add(
            sub, self.timeout,
            "Query or set the timeToIdle for the given session")
        timeout.add_argument(
            "seconds",
            nargs="?",
            type=int,
            help="Number of seconds to set the timeToIdle value to")
        timeout.add_argument(
            "--session",
            help="Session other than the current to update")

        list = parser.add(sub, self.list, (
            "List all available sessions stored locally\n\n" + LISTHELP))
        list.add_argument(
            "--no-purge", dest="purge", action="store_false",
            help="Do not remove inactive sessions")

        who = parser.add(sub, self.who, (
            "List all active server sessions\n\n" + WHOHELP))
        who.add_argument(
            "--show-uuid", help="Show uuids for sessions",
            action="store_true")


        keepalive = parser.add(
            sub, self.keepalive, "Keeps the current session alive")
        keepalive.add_argument(
            "-f", "--frequency", type=int, default=60,
            help="Time in seconds between keep alive calls", metavar="SECS")

        clear = parser.add(
            sub, self.clear, "Close and remove locally stored sessions")
        clear.add_argument(
            "--all", action="store_true",
            help="Remove all locally stored sessions not just inactive ones")

        file = parser.add(
            sub, self.file, "Print the path to the current session file")

        key = parser.add(
            sub, self.key, "Print the key of the current active session")

        for x in (file, key, logout, keepalive, list, clear, group):
            self._configure_dir(x)

        open = parser.add(sub, self.open, "Create a session for "
                                          "the given user and group")
        self.add_single_user_argument(open)
        self.add_single_group_argument(open, required=False)
        open.add_argument("--timeout", nargs="?", type=int, default=0,
                          help="Timeout in seconds (optional; default: "
                               "maximum possible)")

        close = parser.add(sub, self.close, "Close the session with "
                                            "the given session ID")
        close.add_argument("sessionId", nargs="?", help="The session ID")

    def _configure_login(self, login):
        login.add_login_arguments()
        login.add_argument(
            "-t", "--timeout", type=int,
            help="Timeout for session. After this many inactive seconds, the"
            " session will be closed")
        login.add_argument(
            "--retry", nargs="?", type=int, default=0,
            help="Number of seconds to retry login (default: no retry)")
        login.add_argument(
            "connection", nargs="?",
            help="Connection string. See extended help for examples")
        self._configure_dir(login)

    def _configure_dir(self, parser):
        parser.add_argument("--session-dir", help=SUPPRESS)

    def help(self, args):
        self.ctx.out(LONGHELP % {"prog": args.prog})

    @admin_only(AdminPrivilegeSudo)
    def open(self, args):
        client = self.ctx.conn(args)
        admin = client.sf.getAdminService()

        user, group = self.get_single_user_group(args, admin)
        if not user:
            self.ctx.die(156, "No user found")
        username = user.omeName.val
        groupname = None
        if group:
            groupname = group.name.val

        p = omero.sys.Principal()
        p.name = username
        if group:
            p.group = groupname
        p.eventType = "User"
        svc = client.sf.getSessionService()
        sess = svc.createSessionWithTimeout(p, (int(args.timeout)
                                                * 1000))
        sessId = sess.getUuid().val
        tti = old_div(sess.getTimeToIdle().val, 1000)
        ttl = old_div(sess.getTimeToLive().val, 1000)

        msg = "Session created for user %s" % username
        if groupname:
            msg += " in group %s" % groupname
        msg += " (timeToIdle: %d sec, timeToLive: %d sec)" % (tti, ttl)

        self.ctx.err(msg)
        self.ctx.out(sessId)  # Ok, must share

    def close(self, args):
        client = self.ctx.conn(args)
        svc = client.sf.getSessionService()
        session = None
        try:
            session = svc.getSession(args.sessionId)
        except Exception:
            self.ctx.err("No session with the given ID found.")

        if session:
            client.destroySession(args.sessionId)
            self.ctx.out("Session %s closed." % args.sessionId)

    def login(self, args):
        ("Login to a given server, and store session key locally.\n\n"
         "USER, HOST, and PORT are set as args or in a ssh-style "
         "connection string.\n"
         "PASSWORD can be entered interactively, or passed via "
         "-w (insecure!).\n"
         "Alternatively, a session KEY can be passed with '-k'.\n"
         "Admin users can use --sudo=ADMINUSER to login for others.\n\n"
         "Examples:\n"
         "  omero login example.com\n"
         "  omero login user@example.com\n"
         "  omero login user@example.com:24064\n"
         "  omero login -k SESSIONKEY example.com\n"
         "  omero login --sudo=root user@example\n"
         "\n")

        """
        Goals:
        If server and key, then don't ask any questions.
        If nothing requested, and something's active, use it. (i.e. don't
        require port number)
        Reconnect if possible (assuming parameters are the same)
        """

        if self.ctx.conn() and not self.ctx.isquiet:
            self.ctx.err("Active client found")
            return  # EARLY EXIT

        create = getattr(args, "create", None)
        store = self.store(args)
        previous = store.get_current()

        # Basic props, don't get fiddled with
        props = {}
        if args.group:
            props["omero.group"] = args.group

        #
        # Retrieving the parameters as set by the user
        # If these are set and different from the current
        # connection, then a new one may be created.
        #
        # May be called by another plugin
        server = getattr(args, "connection", None)
        name = None
        port = None

        if args.server:
            if server:
                self.ctx.die(3, "Server specified twice: %s and %s"
                             % (server, args.server))
            else:
                server = args.server

        if server:
            server, name, port = self._parse_conn(server, name)

        if args.user:
            if name:
                self.ctx.die(4, "Username specified twice: %s and %s"
                             % (name, args.user))
            else:
                name = args.user

        if args.port:
            if port:
                self.ctx.die(5, "Port specified twice: %s and %s"
                             % (port, args.port))
            else:
                port = args.port

        #
        # If a key is provided, then that takes precedence.
        # Unless the key is bad, there's no reason to ask
        # the user for any more input.
        #
        pasw = args.password
        if args.key:
            if name and not self.ctx.isquiet:
                self.ctx.err("Overriding name since session key set")
            name = args.key
            if args.group and not self.ctx.isquiet:
                self.ctx.err("Ignoring group since session key set")
            if args.password and not self.ctx.isquiet:
                self.ctx.err("Ignoring password since session key set")
            pasw = args.key
        #
        # If no key provided, then we check the last used connection
        # by default. The only requirement is that the user can't
        # have requested a differente server / name or conflicting
        # props (group / port)
        #
        elif previous[0] and previous[1]:

                server_differs = (server is not None and server != previous[0])
                name_differs = (name is not None and name != previous[1])
                port_differs = (port is not None and port != previous[3])

                if not create and not server_differs and not name_differs \
                        and not port_differs:
                    try:
                        if previous[2] is not None:
                            # Missing session uuid file. Deleted? See #4199
                            conflicts = store.conflicts(
                                previous[0], previous[1], previous[2], props,
                                True)
                            if conflicts:
                                self.ctx.dbg("Not attaching because of"
                                             " conflicts: %s" % conflicts)
                            else:
                                rv = store.attach(*previous[:-1])
                                return self.handle(rv, "Using")
                        if not self.ctx.isquiet:
                            self.ctx.out("Previously logged in to %s:%s as %s"
                                         % (previous[0], previous[3],
                                            previous[1]))
                    except Exception as e:
                        self.ctx.out("Previous session expired for %s on"
                                     " %s:%s" % (previous[1], previous[0],
                                                 previous[3]))
                        self.ctx.dbg("Exception on attach: %s"
                                     % traceback.format_exception(None, e, sys.exc_info()[2]))
                        try:
                            store.remove(*previous[:-1])
                        except OSError as ose:
                            self.ctx.dbg("Session file missing: %s" % ose)
                        except:
                            self.ctx.dbg("Exception on remove: %s"
                                         % traceback.format_exception(None, e, sys.exc_info()[2]))
                            # Could tell user to manually clear here and then
                            # self.ctx.die()
                            self.ctx.err("Failed to remove session: %s" % e)

        #
        # If we've reached here, then the user either does not have
        # an active session or has requested another (different options)
        # If they've omitted some required value, we must ask for it.
        #
        if not server:
            server, name, port = self._get_server(store, name, port)
        if not name:
            name = self._get_username(previous[1])

        props["omero.host"] = server
        props["omero.user"] = name
        if port:
            props["omero.port"] = port
        if "timeout" in args and args.timeout:
            props["omero.timeout"] = args.timeout

        rv = None
        #
        # For session key access, we now need to lookup the stored_name
        # since otherwise, all access to the directory under ~/omero/sessions
        # will fail. Then, if no session can be found, we exit immediately
        # rather than asking for a password. See #4223
        #
        if args.key:
            stored_name = store.find_name_by_key(server, args.key)
            if not stored_name:
                # ticket:5975 : If this is the case, then this session key
                # did not come from a CLI login, and so we're not going to
                # modify the value returned by store.get_current()
                self.ctx.dbg("No local session file found for %s." % args.key)
                rv = self.attach(store, server, args.key, args.key, props,
                                 False, set_current=False)
            else:
                rv = self.check_and_attach(store, server, stored_name,
                                           args.key, props, check_group=False)
            action = "Joined"
            if not rv:
                if port:
                    msg = "Cannot join %s on %s:%s." % (args.key, server, port)
                else:
                    msg = "Cannot join %s on %s." % (args.key, server)
                self.ctx.die(523, "Bad session key. %s" % msg)
        elif not create:
            available = store.available(server, name)
            for uuid in available:
                rv = self.check_and_attach(store, server, name, uuid, props,
                                           check_group=True)
                action = "Reconnected to"

        if not rv:

            if not pasw:
                pasw = os.getenv("OMERO_PASSWORD")

            tries = 3
            while True:
                try:
                    if not pasw:
                        # Handle absent and incorrect passwords in
                        # non-interactive mode
                        self._require_tty("cannot request password")
                        if args.sudo:
                            prompt = "Password for %s:" % args.sudo
                        else:
                            prompt = "Password:"
                        pasw = self.ctx.input(prompt, hidden=True,
                                              required=True)

                    ### Retry logic ############################################
                    start = time.time()
                    retries = 0
                    while True:
                        try:
                            rv = store.create(name, pasw, props, sudo=args.sudo)
                            break
                        except Exception as e:
                            elapsed = (time.time() - start)
                            cannot_retry = (
                                'retry' not in vars(args) or
                                not args.retry or
                                elapsed > args.retry)
                            if cannot_retry:
                                raise
                            else:
                                retries += 1
                                msg = str(datetime.datetime.now().time())
                                msg += ": Login retry #%d in 3s" % (retries)
                                self.ctx.err(msg)
                                time.sleep(3)
                    ### End retry ##############################################

                    break
                except PermissionDeniedException as pde:
                    tries -= 1
                    if not tries:
                        self.ctx.die(524, "3 incorrect password attempts")
                    else:
                        self.ctx.err(pde.reason)
                        pasw = None
                except omero.RemovedSessionException as rse:
                    self.ctx.die(525, "User account error: %s." % rse.message)
                except Ice.ConnectionRefusedException:
                    if port:
                        self.ctx.die(554, "Ice.ConnectionRefusedException:"
                                     " %s:%s isn't running" % (server, port))
                    else:
                        self.ctx.die(554, "Ice.ConnectionRefusedException: %s"
                                     " isn't running" % server)
                except Ice.DNSException:
                    self.ctx.die(555, "Ice.DNSException: bad host name: '%s'"
                                 % server)
                except omero.SecurityViolation as sv:
                    self.ctx.die(557, "SecurityViolation: %s" % sv.message)
                except Exception as e:
                    exc = traceback.format_exc()
                    self.ctx.dbg(exc)
                    self.ctx.die(556, "InternalException: Failed to connect:"
                                 " %s" % e)
            action = "Created"

        return self.handle(rv, action)

    def check_and_attach(self, store, server, name, uuid, props,
                         check_group=False):
        """
        Checks for conflicts in the settings for this session,
        and if there are none, then attempts an "attach()". If
        that fails, the session is removed.
        """

        exists = store.exists(server, name, uuid)

        if exists:
            conflicts = store.conflicts(server, name, uuid, props,
                                        check_group=check_group)
            if conflicts:
                if "omero.port" in conflicts:
                    self.ctx.dbg("Skipping session %s due to mismatching"
                                 " ports: %s " % (uuid, conflicts))
                elif not self.ctx.isquiet:
                    self.ctx.err("Skipped session %s due to property"
                                 " conflicts: %s" % (uuid, conflicts))
                return None

        return self.attach(store, server, name, uuid, props, exists)

    def attach(self, store, server, name, uuid, props, exists,
               set_current=True):
        rv = None
        try:
            if exists:
                rv = store.attach(server, name, uuid, set_current=set_current)
            else:
                rv = store.create(name, name, props, set_current=set_current)
        except Exception as e:
            self.ctx.dbg("Removing %s: %s" % (uuid, e))
            store.clear(server, name, uuid)
        return rv

    def handle(self, rv, action):
        """
        Handles a new connection
        """
        client, uuid, idle, live = rv
        sf = client.sf

        # detachOnDestroy called by omero.util.sessions
        client.enableKeepAlive(300)
        ec = sf.getAdminService().getEventContext()
        self.ctx.set_event_context(ec)
        self.ctx.set_client(client)

        host = client.getProperty("omero.host")
        port = client.getProperty("omero.port")

        msg = "%s session for %s@%s:%s." \
            % (action, ec.userName, host, port)
        msg += self._parse_timeout(idle, " Idle timeout: ")
        msg += self._parse_timeout(live, " Expires in : ")

        msg += (" Current group: %s" % ec.groupName)

        if not self.ctx.isquiet:
            self.ctx.err(msg)

    def _parse_timeout(self, timeout, msg=""):
        timeout = unwrap(timeout)
        if not timeout:
            return ""

        unit = "min."
        val = float(timeout) / 60 / 1000
        if val < 5:
            unit = "s."
            val = val * 60
        return "%s%.f %s" % (msg, val, unit)

    def logout(self, args):
        store = self.store(args)
        previous = store.get_current()

        try:
            rv = store.attach(*previous[:-1])
            rv[0].killSession()
        except Exception as e:
            self.ctx.dbg("Exception on logout: %s" % e)
        store.remove(*previous[:-1])
        # Last is still useful. Not resetting.
        # store.set_current("", "", "")

    def group(self, args):
        client = self.ctx.conn(args)
        sf = client.sf
        admin = sf.getAdminService()

        if args.target is None:
            ec = self.ctx.get_event_context()
            self.ctx.out("ExperimenterGroup:%s" % ec.groupId)
            return ec.groupName

        try:
            group_id = int(args.target)
            group_name = admin.getGroup(group_id).name.val
        except ValueError:
            group_name = args.target
            group_id = admin.lookupGroup(group_name).id.val

        ec = self.ctx.get_event_context()  # 5711
        old_id = ec.groupId
        old_name = ec.groupName
        if old_id == group_id and not self.ctx.isquiet:
            self.ctx.err("Group '%s' (id=%s) is already active"
                         % (group_name, group_id))
        else:
            try:
                sf.setSecurityContext(omero.model.ExperimenterGroupI(
                    group_id, False))
                self.ctx.set_event_context(
                    sf.getAdminService().getEventContext())
                self.ctx.out("Group '%s' (id=%s) switched to '%s' (id=%s)" % (
                    old_name, old_id, group_name, group_id))
            except omero.SecurityViolation as sv:
                    self.ctx.die(564, "SecurityViolation: %s" % sv.message)

    def timeout(self, args):
        client = self.ctx.conn(args)
        svc = client.sf.getSessionService()

        uuid = args.session
        if uuid is None:
            uuid = self.ctx.get_event_context().sessionUuid
        try:
            obj = svc.getSession(uuid)
        except:
            self.ctx.dbg(traceback.format_exc())
            self.ctx.die(557, "cannot get session: %s" % uuid)

        if args.seconds is None:
            # Query only
            secs = old_div(unwrap(obj.timeToIdle),1000.0)
            self.ctx.out(secs)
            return secs

        req = omero.cmd.UpdateSessionTimeoutRequest()
        req.session = uuid
        req.timeToIdle = rlong(args.seconds * 1000)
        try:
            cb = client.submit(req)  # Response is "OK"
            cb.close(True)
        except omero.CmdError as ce:
            self.ctx.dbg(str(ce.err))
            self.ctx.die(558, "CmdError: %s" % ce.err.name)
        except:
            self.ctx.dbg(traceback.format_exc())
            self.ctx.die(559, "cannot update timeout for %s" % uuid)

    def list(self, args):
        store = self.store(args)
        s = store.contents()
        previous = store.get_current()

        headers = ("Server", "User", "Group", "Session", "Active", "Started")
        results = dict([(x, []) for x in headers])
        for server, names in list(s.items()):
            for name, sessions in list(names.items()):
                for uuid, props in list(sessions.items()):
                    rv = None
                    msg = "True"
                    grp = "Unknown"
                    started = "Unknown"
                    port = None
                    try:
                        if props:
                            port = props.get("omero.port", port)
                        rv = store.attach(server, name, uuid)
                        try:
                            a_s = rv[0].sf.getAdminService()
                            grp = a_s.getEventContext().groupName
                            s_s = rv[0].sf.getSessionService()
                            started = s_s.getSession(uuid).started.val
                            started = time.ctime(old_div(started, 1000.0))
                        finally:
                            if rv:
                                rv[0].closeSession()
                    except PermissionDeniedException as pde:
                        msg = pde.reason
                    except Exception as e:
                        self.ctx.dbg("Exception on attach: %s" % e)
                        msg = "Unknown exception"

                    if rv is None and args.purge:
                        try:
                            self.ctx.dbg("Purging %s / %s / %s"
                                         % (server, name, uuid))
                            store.remove(server, name, uuid)
                            continue
                        except IOError as ioe:
                            self.ctx.dbg("Aborting session purging. %s" % ioe)
                            break

                    if server == previous[0] and name == previous[1] and \
                            uuid == previous[2]:
                        msg = "Logged in"

                    if port:
                        results["Server"].append("%s:%s" % (server, port))
                    else:
                        results["Server"].append(server)

                    results["User"].append(name)
                    results["Group"].append(grp)
                    results["Session"].append(uuid)
                    results["Active"].append(msg)
                    results["Started"].append(started)

        from omero.util.text import Table, Column
        columns = tuple([Column(x, results[x]) for x in headers])
        self.ctx.out(str(Table(*columns)))

    def who(self, args):
        show_uuid = args.show_uuid
        client = self.ctx.conn(args)
        uuid = self.ctx.get_event_context().sessionUuid
        req = omero.cmd.CurrentSessionsRequest()
        try:
            cb = client.submit(req)
            try:
                rsp = cb.getResponse()
            finally:
                cb.close(True)

            headers = ["name", "group", "logged in", "agent", "timeout"]
            extra = set()
            results = {"name": [], "group": [],
                       "logged in": [], "agent": [],
                       "timeout": [], "uuid": []}

            # Preparse data to find extra columns
            for idx, s in enumerate(rsp.sessions):
                for k in list(rsp.data[idx].keys()):
                    extra.add(k)
            if show_uuid:
                headers.append("uuid")
            for add in sorted(extra):
                headers.append(add)
                results[add] = []

            for idx, s in enumerate(rsp.sessions):
                ec = rsp.contexts[idx]
                data = unwrap(rsp.data[idx])
                # Handle missing keys
                for k in extra:
                    if k not in list(data.keys()):
                        results[k].append("---")
                for k, v in sorted(data.items()):
                    try:
                        if k.endswith("Time"):
                            t = old_div(v, 1000.0)
                            t = time.localtime(t)
                            v = time.strftime('%Y-%m-%d %H:%M:%S', t)
                    except:
                        pass
                    results[k].append(v)
                results["name"].append(ec.userName)
                results["group"].append(ec.groupName)
                if s is not None:
                    t = old_div(s.started.val, 1000.0)
                    t = time.localtime(t)
                    t = time.strftime("%Y-%m-%d %H:%M:%S", t)
                    if uuid == ec.sessionUuid:
                        t = t + " (*)"
                    results["logged in"].append(t)
                    results["agent"].append(unwrap(s.userAgent))
                    results["timeout"].append(
                        self._parse_timeout(s.timeToIdle))
                    results["uuid"].append(ec.sessionUuid)
                else:
                    # Insufficient privileges. The EventContext
                    # will be missing fields as well.
                    msg = "---"
                    results["logged in"].append(msg)
                    results["agent"].append(msg)
                    results["timeout"].append(msg)
                    results["uuid"].append(msg)

            from omero.util.text import Table, Column
            columns = tuple([Column(x, results[x]) for x in headers])
            self.ctx.out(str(Table(*columns)))
        except omero.CmdError as ce:
            self.ctx.dbg(str(ce.err))
            self.ctx.die(560, "CmdError: %s" % ce.err.name)
        except omero.ClientError as ce:
            if ce.message == "Null handle":
                v = client.sf.getConfigService().getVersion()
                self.ctx.die(561,
                             "Operation unsupported. Server version: %s" % v)
            else:
                exc = traceback.format_exc()
                self.ctx.dbg(exc)
                self.ctx.die(562, "ClientError: %s" % ce.err.name)
        except omero.LockTimeout:
            exc = traceback.format_exc()
            self.ctx.dbg(exc)
            self.ctx.die(563, "LockTimeout: operation took too long")

    def clear(self, args):
        store = self.store(args)
        count = store.count()
        store.clear()
        self.ctx.out("%s session(s) cleared" % count)

    def keepalive(self, args):
        import threading
        from omero.util.concurrency import get_event as get_event

        class T(threading.Thread):
            def run(self):
                while self.client:
                    try:
                        self.client.sf.keepAlive(None)
                        self.event.wait(args.frequency)
                    except Exception as e:
                        self.err("Keep alive failed: %s" % str(e))
                        return
        t = T()
        t.client = self.ctx.conn(args)
        t.err = self.ctx.err
        t.event = get_event(name="keepalive")
        t.start()
        try:
            self.ctx.out("Running keep alive every %s seconds"
                         % args.frequency)
            self.ctx.input("Press enter to cancel.")
        finally:
            t.client = None
            t.event.set()

    def file(self, args):
        """Return the file associated with the current active session"""
        store = self.store(args)
        srv, usr, uuid, port = store.get_current()
        if srv and usr and uuid:
            self.ctx.out(str(store.dir / srv / usr / uuid))

    def key(self, args):
        """Return the key associated with the current active session"""
        store = self.store(args)
        srv, usr, uuid, port = store.get_current()
        if uuid:
            self.ctx.out(uuid)

    def conn(self, properties=None, profile=None, args=None):
        """
        Either creates or returns the exiting omero.client instance.
        Uses the comm() method with the same signature.
        """

        if properties is None:
            properties = {}

        if self.get_client():
            return self.get_client()

        import omero
        try:
            data = self.initData(properties)
            self.set_client(omero.client(sys.argv, id=data))
            self.get_client().setAgent("OMERO.cli")
            self.get_client().createSession()
            return self.get_client()
        except Exception:
            self.set_client(None)
            raise

    #
    # Private methods
    #

    @staticmethod
    def _parse_conn(server, default_name):
        """Parse a connection string of form (user@)server(:port)"""

        import re
        pat = r'^((?P<name>.+)@)?(?P<server>.*?)(:(?P<port>\d{1,5}))?$'
        match = re.match(pat, server)
        server = match.group('server')
        name = match.group('name')
        port = match.group('port')
        if not name:
            name = default_name
        return server, name, port

    def _get_server(self, store, name, port):
        defserver = store.last_host()
        if not port:
            port = store.last_port()
        self._require_tty("cannot request server")
        rv = self.ctx.input("Server: [%s:%s]" % (defserver, port))
        if not rv:
            return defserver, name, port
        else:
            return self._parse_conn(rv, name)

    def _get_username(self, defuser):
        if defuser is None:
            defuser = get_user("root")
        self._require_tty("cannot request username")
        rv = self.ctx.input("Username: [%s]" % defuser)
        if not rv:
            return defuser
        else:
            return rv

    def _require_tty(self, msg):
        if not sys.stdin.isatty():
            self.ctx.die(564, "stdin is not a terminal: %s" % msg)
        elif not sys.stdout.isatty():
            self.ctx.die(564, "stdout is not a terminal: %s" % msg)
        else:
            return


try:
    register("sessions", SessionsControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("sessions", SessionsControl, HELP)
        cli.invoke(sys.argv[1:])



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""

[hdr]




# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `NamedValue.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module omero
_M_omero = Ice.openModule('omero')
__name__ = 'omero'

# Start of module omero.model
_M_omero.model = Ice.openModule('omero.model')
__name__ = 'omero.model'

if 'NamedValue' not in _M_omero.model.__dict__:
    _M_omero.model.NamedValue = Ice.createTempClass()
    class NamedValue(Ice.Object):
        """
        Simple Pair-like container which is
        used in a sequence to support ordered maps.
        """
        def __init__(self, name='', value=''):
            self.name = name
            self.value = value

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::NamedValue')

        def ice_id(self, current=None):
            return '::omero::model::NamedValue'

        def ice_staticId():
            return '::omero::model::NamedValue'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_NamedValue)

        __repr__ = __str__

    _M_omero.model.NamedValuePrx = Ice.createTempClass()
    class NamedValuePrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.NamedValuePrx.ice_checkedCast(proxy, '::omero::model::NamedValue', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.NamedValuePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::NamedValue'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_NamedValuePrx = IcePy.defineProxy('::omero::model::NamedValue', NamedValuePrx)

    _M_omero.model._t_NamedValue = IcePy.defineClass('::omero::model::NamedValue', NamedValue, -1, (), False, False, None, (), (
        ('name', (), IcePy._t_string, False, 0),
        ('value', (), IcePy._t_string, False, 0)
    ))
    NamedValue._ice_type = _M_omero.model._t_NamedValue

    _M_omero.model.NamedValue = NamedValue
    del NamedValue

    _M_omero.model.NamedValuePrx = NamedValuePrx
    del NamedValuePrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2010 Glencoe Software, Inc. All rights reserved.
#   Use is subject to license terms supplied in LICENSE.txt
#

import IceImport
IceImport.load("omero_FS_ice")
IceImport.load("omero_Scripts_ice")
IceImport.load("omero_SharedResources_ice")
IceImport.load("omero_Tables_ice")
